from pyxel import _get_constant_number, _get_constant_string

APP_WIDTH = 240
APP_HEIGHT = 180

EDITOR_IMAGE_X = 0
EDITOR_IMAGE_Y = 32
EDITOR_IMAGE_NAME = "editor_220x160.png"

TILEMAP_IMAGE_X = 0
TILEMAP_IMAGE_Y = 192
TILEMAP_IMAGE_WIDTH = 64
TILEMAP_IMAGE_HEIGHT = 63

TOOL_SELECT = 0
TOOL_PENCIL = 1
TOOL_RECTB = 2
TOOL_RECT = 3
TOOL_CIRCB = 4
TOOL_CIRC = 5
TOOL_BUCKET = 6

MAX_SOUND_LENGTH = 48
MAX_MUSIC_LENGTH = 32

IMAGE_BANK_COUNT = _get_constant_number("IMAGE_BANK_COUNT")
TILEMAP_BANK_COUNT = _get_constant_number("TILEMAP_BANK_COUNT")
SOUND_BANK_COUNT = _get_constant_number("SOUND_BANK_COUNT")
MUSIC_BANK_COUNT = _get_constant_number("MUSIC_BANK_COUNT")
MUSIC_CHANNEL_COUNT = _get_constant_number("MUSIC_CHANNEL_COUNT")
FONT_WIDTH = _get_constant_number("FONT_WIDTH")
FONT_HEIGHT = _get_constant_number("FONT_HEIGHT")

RESOURCE_FILE_EXTENSION = _get_constant_string("RESOURCE_FILE_EXTENSION")
